@@ -7,7 +7,7 @@ Examples
 --------
 Create a radial profile in the xy-plane.
 
->>> p = Profile(snap, radius_min=10, radius_max=200, n_bins=100)
+>>> p = Profile(snap, cmin=10, cmax=200, n_bins=100)
 >>> p.plot('radius', 'density')
 
 Calculate the angular momentum on the particles.
