@@ -4,7 +4,7 @@ The Plonk implementation for visualizing smoothed particle hydrodynamics
 simulations using kernel density estimation based interpolation.
 """
 
-from .animation import animation, animation_profiles
+from .animation import animation, animation_particles, animation_profiles
 from .functions import get_extent_from_percentile, str_to_units
 from .interpolation import interpolate
 from .multi import plot_snaps
@@ -12,6 +12,7 @@ from .visualization import particle_plot, plot
 
 __all__ = [
     'animation',
+    'animation_particles',
     'animation_profiles',
     'get_extent_from_percentile',
     'interpolate',
